@@ -3,6 +3,7 @@
 
 """
 ===============================================================================
+
     Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
@@ -18,6 +19,7 @@
     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
     See the License for the specific language governing permissions and
     limitations under the License.
+
 ===============================================================================
 """
 
@@ -36,7 +38,7 @@ import colander
 from colander import (
     Boolean,
     Date,
-    DateTime,
+    DateTime,  # NB name clash with pendulum
     Email,
     Integer,
     Invalid,
@@ -51,7 +53,7 @@ from deform.widget import (
     DateTimeInputWidget,
     HiddenWidget,
 )
-from pendulum import Pendulum
+from pendulum import DateTime as Pendulum  # NB name clash with colander
 from pendulum.parsing.exceptions import ParserError
 
 if TYPE_CHECKING:
@@ -102,8 +104,10 @@ class PendulumType(SchemaType):
             appstruct = coerce_to_pendulum(appstruct,
                                            assume_local=self.use_local_tz)
         except (ValueError, ParserError) as e:
-            raise Invalid(node, "{!r} is not a Pendulum object; error was "
-                                "{!r}".format(appstruct, e))
+            raise Invalid(
+                node,
+                "{!r} is not a pendulum.DateTime object; error was "
+                "{!r}".format(appstruct, e))
         return appstruct.isoformat()
 
     def deserialize(self,
